package com.capgemini.reports.templates;


import org.testng.annotations.Test;

import static org.assertj.core.api.Assertions.assertThat;

public class ColumnTest {

  private static final int CONTENT_HEIGHT = 13;
  private static final int HEADER_HEIGHT = 20;
  private static final int EXTRA_HEIGHT = 10;
  private static final String HEADER_KEY = "HEADER_KEY";
  private static final String CONTENT_KEY = "CONTENT_KEY";
  private static final int WIDTH = 100;
  private static final int XPOS = 0;
  private static final String COLOR = "#E3E3E3";

  private static final String EXPECTED_CONTENT = "" /*------------------------------------------------------------------------------------------*/
      + "\n            <textField>" /*------------------------------------------------------------------------------------------------------*/
      + "\n                <reportElement x=\"" + XPOS + "\" y=\"0\" width=\"" + WIDTH + "\" height=\"" + CONTENT_HEIGHT + "\"/>" /*--------*/
      + "\n                <box leftPadding=\"5\">"/*---------------------------------------------------------------------------------------*/
      + "\n                    <topPen lineWidth=\"0.5\"/>" /*------------------------------------------------------------------------------*/
      + "\n                    <leftPen lineWidth=\"0.5\"/>" /*-----------------------------------------------------------------------------*/
      + "\n                    <bottomPen lineWidth=\"0.5\"/>"/*----------------------------------------------------------------------------*/
      + "\n                    <rightPen lineWidth=\"0.5\"/>" /*----------------------------------------------------------------------------*/
      + "\n                </box>" /*-------------------------------------------------------------------------------------------------------*/
      + "\n                <textElement markup=\"html\">" /*-------------------------------------------------------------------------------*/
      + "\n\n                </textElement>"/*-------------------------------------------------------------------------------------------------*/
      + "\n                <textFieldExpression><![CDATA[$F{" + CONTENT_KEY + "}]]></textFieldExpression>"/*--------------------------------*/
      + "\n            </textField>";

  private static final String EXPECTED_HEADER = ""/*--------------------------------------------------------------------------------------------*/
      + "\n            <staticText>"/*------------------------------------------------------------------------------------------------------*/
      + "\n                <reportElement x=\"" + XPOS + "\" y=\"0\" width=\"" + WIDTH + "\" height=\"" + HEADER_HEIGHT + "\"/>" /*---------*/
      + "\n                <box leftPadding=\"5\">" /*--------------------------------------------------------------------------------------*/
      + "\n                    <topPen lineWidth=\"0.5\"/>"/*-------------------------------------------------------------------------------*/
      + "\n                    <leftPen lineWidth=\"0.5\"/>" /*-----------------------------------------------------------------------------*/
      + "\n                    <bottomPen lineWidth=\"0.5\"/>"/*----------------------------------------------------------------------------*/
      + "\n                    <rightPen lineWidth=\"0.5\"/>" /*----------------------------------------------------------------------------*/
      + "\n                </box>" /*-------------------------------------------------------------------------------------------------------*/
      + "\n                <textElement markup=\"html\">" /*-------------------------------------------------------------------------------*/
      + "\n\n                </textElement>"/*-------------------------------------------------------------------------------------------------*/
      + "\n                <text><![CDATA[" + HEADER_KEY + "]]></text>"/*-------------------------------------------------------------------*/
      + "\n            </staticText>";

  private static final String EXPECTED_GREY_HEADER = ""/*---------------------------------------------------------------------------------------*/
      + "\n            <staticText>" /*-----------------------------------------------------------------------------------------------------*/
      + "\n                <reportElement x=\"" + XPOS + "\" y=\"0\" width=\"" + WIDTH + "\" height=\"" + HEADER_HEIGHT
      + "\" mode=\"Opaque\" backcolor=\"" + COLOR + "\"/>"/*-------------------------------------------------------------------------------------------*/
      + "\n                <box leftPadding=\"5\">"/*---------------------------------------------------------------------------------------*/
      + "\n                    <topPen lineWidth=\"0.5\"/>"/*-------------------------------------------------------------------------------*/
      + "\n                    <leftPen lineWidth=\"0.5\"/>" /*-----------------------------------------------------------------------------*/
      + "\n                    <bottomPen lineWidth=\"0.5\"/>"/*----------------------------------------------------------------------------*/
      + "\n                    <rightPen lineWidth=\"0.5\"/>" /*----------------------------------------------------------------------------*/
      + "\n                </box>" /*-------------------------------------------------------------------------------------------------------*/
      + "\n                <textElement markup=\"html\">" /*-------------------------------------------------------------------------------*/
      + "\n\n                </textElement>"/*-------------------------------------------------------------------------------------------------*/
      + "\n                <text><![CDATA[" + HEADER_KEY + "]]></text>" /*------------------------------------------------------------------*/
      + "\n            </staticText>";

  private static final String EXPECTED_EXTRA_HIGH_HEADER = "" /*--------------------------------------------------------------------------------*/
      + "\n            <staticText>" /*-----------------------------------------------------------------------------------------------------*/
      + "\n                <reportElement x=\"" + XPOS + "\" y=\"0\" width=\"" + WIDTH + "\" height=\"" + (HEADER_HEIGHT + EXTRA_HEIGHT)
      + "\"/>" /*---------------------------------------------------------------------------------------------------------------------------*/
      + "\n                <box leftPadding=\"5\">" /*--------------------------------------------------------------------------------------*/
      + "\n                    <topPen lineWidth=\"0.5\"/>" /*------------------------------------------------------------------------------*/
      + "\n                    <leftPen lineWidth=\"0.5\"/>" /*-----------------------------------------------------------------------------*/
      + "\n                    <bottomPen lineWidth=\"0.5\"/>" /*---------------------------------------------------------------------------*/
      + "\n                    <rightPen lineWidth=\"0.5\"/>"/*-----------------------------------------------------------------------------*/
      + "\n                </box>" /*-------------------------------------------------------------------------------------------------------*/
      + "\n                <textElement markup=\"html\">" /*-------------------------------------------------------------------------------*/
      + "\n\n                </textElement>"/*-------------------------------------------------------------------------------------------------*/
      + "\n                <text><![CDATA[" + HEADER_KEY + "]]></text>" /*------------------------------------------------------------------*/
      + "\n            </staticText>"; /*---------------------------------------------------------------------------------------------------*/

  @Test
  public void testColumnField() {
    Column col = new Column(CONTENT_KEY, WIDTH, XPOS, false, CONTENT_HEIGHT, null);
    assertThat(col.makeContent()).isEqualTo(EXPECTED_CONTENT);
    assertThat(col.makeHeaders()).isEmpty();
  }

  @Test
  public void testAddHeader() {
    Column col = new Column(CONTENT_KEY, WIDTH, XPOS, false, HEADER_HEIGHT, null);
    String headerStr = col.makeHeaders();
    assertThat(headerStr).isEmpty();
    col.addHeader(HEADER_KEY, false, HEADER_HEIGHT, false, null);
    headerStr = col.makeHeaders();
    assertThat(headerStr).isEqualTo(EXPECTED_HEADER);
  }

  @Test
  public void testGreyHeader() {
    Column col = new Column(CONTENT_KEY, WIDTH, XPOS, false, HEADER_HEIGHT, null);
    col.addHeader(HEADER_KEY, false, HEADER_HEIGHT, false, new CellFormat().setColor(COLOR));
    String headerStr = col.makeHeaders();
    assertThat(headerStr).isEqualTo(EXPECTED_GREY_HEADER);
  }

  @Test
  public void testAddToPreviousHeaderCellHeight() {
    Column col = new Column(CONTENT_KEY, WIDTH, XPOS, false, HEADER_HEIGHT, null);
    col.addHeader(HEADER_KEY, false, HEADER_HEIGHT, false, null);
    col.addToPreviousHeaderCellHeight(EXTRA_HEIGHT);
    assertThat(col.makeHeaders()).isEqualTo(EXPECTED_EXTRA_HIGH_HEADER);
  }

  @Test
  public void testGetParametersDeclaration() {
    // No parameter yet (No header)
    Column col = new Column(CONTENT_KEY, WIDTH, XPOS, false, HEADER_HEIGHT, null);
    assertThat(col.getParametersDeclaration()).isEmpty();

    // Still no parameters (header is static)
    col.addHeader(HEADER_KEY, false, HEADER_HEIGHT, false, null);
    assertThat(col.getParametersDeclaration()).isEmpty();

    // Now there is a parameter
    col.addHeader(HEADER_KEY, true, HEADER_HEIGHT, false, null);
    String parametersDeclaration = col.getParametersDeclaration();
    assertThat(parametersDeclaration).isEqualTo("\n    <parameter name=\"" + HEADER_KEY + "\" class=\"java.lang.String\"/>");
  }
}
