package com.capgemini.reports.definition.r07report;

import com.capgemini.framework.teamcenter.authent.TcSession;
import com.capgemini.framework.teamcenter.exceptions.InvalidConfigurationException;
import com.capgemini.framework.teamcenter.exceptions.KurtException;
import com.capgemini.framework.teamcenter.exceptions.NoTeamcenterConnectionException;
import com.capgemini.framework.teamcenter.policies.PolicyLocation;
import com.capgemini.reports.builder.R07Builder;
import com.capgemini.reports.database.javabean.enumeration.ReportCriteriaEnum;
import com.capgemini.reports.definition.r07report.data.DataReportPreparatorR07;
import com.capgemini.reports.execution.strategies.ImmediateExecutionStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.assertj.core.api.SoftAssertions;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Optional;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;


@Slf4j
public class Test_R07 {
  private static final TcSession session = new TcSession();
  private SoftAssertions softAssert = new SoftAssertions();
  // Dossier parent du fichier de référence
  private static final String INPUT_FOLDER = "com.capgemini.report.location.default.input";
  // Sous-dossier du fichier de référence
  private static final String SPECIFIC_INPUT_FOLDER = "r07";
  // Dossier de stockage temporaire des fichiers générés initialement
  private static final String TEMPORARY_OUTPUT_FOLDER = "com.capgemini.report.location.default.output";
  // Type de fichier traités
  private static final String SUFFIX = ".xlsx";
  private static final PolicyLocation XML_POLICY = PolicyLocation.make("test_policy", "/xml/policies/r07report/R07_policy_full.xml");
  private Properties localproperties;


  @BeforeClass
  public void login() throws NoTeamcenterConnectionException, InvalidConfigurationException, IOException {
    session.login();
    session.setObjectPropPolicy(XML_POLICY);
    localproperties = new Properties();
    localproperties.load(getClass().getResourceAsStream("/reporting.properties"));
  }

  @AfterClass
  public void logout() {
    session.close();
  }

  //@BeforeMethod
  //TODO : Supprimer xls files dans Generated, checker Tempfile TempDirectory


  @Parameters({"productID", "unit1", "unit2", "generic", "genericAssemb", "refFile"})
  @Test
  public void test07(String productID, String unit1, String unit2,
                     @Optional String generic, @Optional String genericAssemb,
                     String refFile) throws KurtException, IOException {

    // Génération du rapport de test
    Map<ReportCriteriaEnum, String> data = new HashMap<>();
    data.put(ReportCriteriaEnum.PRODUCT, productID);
    data.put(ReportCriteriaEnum.UNIT_CONFIGURATION_1, unit1);
    data.put(ReportCriteriaEnum.UNIT_CONFIGURATION_2, unit2);
    data.put(ReportCriteriaEnum.GENERIC, generic);
    data.put(ReportCriteriaEnum.GENERIC_ASSEMB, genericAssemb);

    ExecutionR07 execution = new R07Builder().build(data);
    new DataReportPreparatorR07().print(session, execution, new ImmediateExecutionStrategy());

    try (FileInputStream fileInput = new FileInputStream(new File((localproperties.getProperty(INPUT_FOLDER)) + SPECIFIC_INPUT_FOLDER + "/" + refFile));
         FileInputStream fileOutput = new FileInputStream(new File(localproperties.getProperty(TEMPORARY_OUTPUT_FOLDER) + execution.getFileNameOnDisc() + SUFFIX))) {

      XSSFWorkbook workbookRef = new XSSFWorkbook(fileInput);
      XSSFWorkbook workbookGen = new XSSFWorkbook(fileOutput);
      DataFormatter df = new DataFormatter();

      softAssert.assertThat(workbookRef.getNumberOfSheets()).isEqualTo(workbookGen.getNumberOfSheets());
      for (int h = 0; h < workbookGen.getNumberOfSheets() && h < workbookRef.getNumberOfSheets(); h++) {
        // Un fichier excel est composé de plusieurs feuilles, on y accède de la manière suivante
        XSSFSheet sheetRef = workbookRef.getSheetAt(h);
        XSSFSheet sheetGen = workbookGen.getSheetAt(h);
        for (int i = 2; i <= sheetRef.getLastRowNum(); i++) {
          XSSFRow rowRef = sheetRef.getRow(i);
          XSSFRow rowGen = sheetGen.getRow(i);
          for (int j = 1; j <= rowRef.getLastCellNum(); j++) {
            XSSFCell cellRef = rowRef.getCell(j);
            XSSFCell cellGen = rowGen.getCell(j);
            softAssert.assertThat(df.formatCellValue(cellGen)).isEqualTo(df.formatCellValue(cellRef));
          }
        }
      }
    }
    softAssert.assertAll();
  }
}
