package com.capgemini.reports.definition.r20report;

import com.capgemini.framework.teamcenter.authent.TcSession;
import com.capgemini.framework.teamcenter.exceptions.InvalidConfigurationException;
import com.capgemini.framework.teamcenter.exceptions.KurtException;
import com.capgemini.framework.teamcenter.exceptions.NoTeamcenterConnectionException;
import com.capgemini.framework.teamcenter.policies.PolicyLocation;
import com.capgemini.reports.builder.R20Builder;
import com.capgemini.reports.database.javabean.enumeration.ReportCriteriaEnum;
import com.capgemini.reports.execution.strategies.ImmediateExecutionStrategy;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.parser.PdfTextExtractor;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.api.SoftAssertions;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Optional;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;


@Slf4j
public class Test_R20 {
  private static final TcSession session = new TcSession();
  private SoftAssertions softAssert = new SoftAssertions();
  // Dossier parent du fichier de référence
  private static final String INPUT_FOLDER = "com.capgemini.report.location.default.input";
  // Sous-dossier du fichier de référence
  private static final String SPECIFIC_INPUT_FOLDER = "r20";
  // Dossier de stockage temporaire des fichiers générés initialement
  private static final String TEMPORARY_OUTPUT_FOLDER = "com.capgemini.report.location.default.output";
  private static final PolicyLocation XML_POLICY = PolicyLocation.make("test_policy", "/xml/policies/r20report/R20_policy_full.xml");
  public static final String SUFFIX = ".pdf";
  public Properties localproperties;


  @BeforeClass
  public void login() throws NoTeamcenterConnectionException, InvalidConfigurationException, IOException {
    session.login();
    session.setObjectPropPolicy(XML_POLICY);
    localproperties = new Properties();
    localproperties.load(getClass().getResourceAsStream("/reporting.properties"));
  }

  @AfterClass
  public void logout() {
    session.close();
  }


  @Test
  @Parameters({"prodIDBefore", "prodIDAfter", "unitBefore", "unitAfter", "matricule", "listModif", "classe", "majMin", "conjoint", "refFile"})
  public void test20(String prodIDBefore, String prodIDAfter, String unitBefore, String unitAfter,
                     String matricule, @Optional String listModif, String classe, String majMin, String conjoint,
                     String refFile) throws KurtException, IOException {

    // Génération du rapport de test
    Map<ReportCriteriaEnum, String> data = new HashMap<>();
    data.put(ReportCriteriaEnum.PRODUCT_BEFORE, prodIDBefore);
    data.put(ReportCriteriaEnum.PRODUCT_AFTER, prodIDAfter);
    data.put(ReportCriteriaEnum.UNIT_BEFORE, unitBefore);
    data.put(ReportCriteriaEnum.UNIT_AFTER, unitAfter);
    data.put(ReportCriteriaEnum.MATRICULE, matricule);
    data.put(ReportCriteriaEnum.LMR, listModif);
    data.put(ReportCriteriaEnum.CLASSE, classe);
    data.put(ReportCriteriaEnum.MAJOR_MINOR, majMin);
    data.put(ReportCriteriaEnum.CONJOINT_UNIQUE, conjoint);


    ExecutionR20 execution = new R20Builder().build(data);
    new DataReportPreparatorR20().print(session, execution, new ImmediateExecutionStrategy());


    PdfReader readerRef = new PdfReader(localproperties.getProperty(INPUT_FOLDER) + SPECIFIC_INPUT_FOLDER + "/" + refFile);
    PdfReader readerGen = new PdfReader(localproperties.getProperty((TEMPORARY_OUTPUT_FOLDER)) + execution.getFileNameOnDisc() + SUFFIX);

    try {
      softAssert.assertThat(readerRef.getNumberOfPages()).isEqualTo(readerGen.getNumberOfPages());
      for (int i = 1; i < readerRef.getNumberOfPages(); i++) {
        String textFromPageRef = new PdfTextExtractor(readerRef).getTextFromPage(i);
        String textFromPageGen = new PdfTextExtractor(readerGen).getTextFromPage(i);
        softAssert.assertThat(textFromPageRef).isEqualTo(textFromPageGen);
      }
    }
    finally {
      readerGen.close();
      readerRef.close();
    }
    softAssert.assertAll();
  }
}
